using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Encompass
{
    public class World
    {
        private readonly ILookup<int, Engine> engineGroups;
        private readonly EntityManager entityManager;
        private readonly ComponentManager componentManager;
        private readonly MessageManager messageManager;
        private readonly RenderManager renderManager;

        internal World(
            ILookup<int, Engine> engineGroups,
            EntityManager entityManager,
            ComponentManager componentManager,
            MessageManager messageManager,
            RenderManager renderManager
        )
        {
            this.engineGroups = engineGroups;
            this.entityManager = entityManager;
            this.componentManager = componentManager;
            this.messageManager = messageManager;
            this.renderManager = renderManager;
        }

        public void Update(double dt)
        {
            foreach (var engineGroup in engineGroups.OrderBy(lookup => lookup.Key))
            {
                try
                {
                    Parallel.ForEach(engineGroup, engine => engine.Update(dt));
                }
                catch (System.AggregateException e)
                {
                    if (e.InnerException != null)
                    {
                        throw e.InnerException;
                    }
                    else
                    {
                        throw e;
                    }
                }
            }

            messageManager.ClearMessages();
            entityManager.DestroyMarkedEntities();

            componentManager.PerformComponentUpdates();
            componentManager.DeactivateMarkedComponents();
            componentManager.RemoveMarkedComponents();

            entityManager.CheckEntitiesWithAddedComponents();
            entityManager.CheckEntitiesWithRemovedComponents();
        }

        public void Draw()
        {
            renderManager.Draw();
        }
    }
}
